from .components.data import get_data_csv, modify_time
from .components.madgwickahrs import MadgwickAHRS
from statistics import mean
import math
from .components.utils import *
import gmplot
import argparse


class Kalman():

    def __init__(self, csv_path=None, output_path=None, mapit=False):
        self.samples = None
        self.csv_path = csv_path
        self.mapit = mapit
        self.output_path = output_path

    def __call__(self, **kwargs):
        try:
            self.get_data(**kwargs)
        except:
            print('Data couldn`t obtained. Process is being continuing without kalman filter')
            return kwargs

        if not self.check_requires():
            return kwargs
        self.samples = len(self.data['accx'])
        print("Interpolating...")

        # !!!Attention!!!  TODO
        # using numpy linear interpolation
        self.gpsTimeStampInterp = np.linspace(self.data['timestamp_ns'][0], self.data['timestamp_ns'][-1],
                                              self.samples)
        self.gpsLatitudeInterpolated = np.interp(self.gpsTimeStampInterp, self.data['timestamp_ns'],
                                                 self.data['lat'])
        self.gpsLongitudeInterpolated = np.interp(self.gpsTimeStampInterp, self.data['timestamp_ns'],
                                                  self.data['lon'])
        # gpsHeadingInterpolated = np.interp(gpsTimeStampInterp, self.data['timestamp_ns'], self.data['heading'])

        # taking GPS time stamps as they GPS is absolute data
        accTimeStampInterp = np.linspace(self.gpsTimeStampInterp[0], self.gpsTimeStampInterp[- 1], self.samples)
        accXInterpolated = np.interp(accTimeStampInterp, self.data['timestamp_ns'], self.data['accx'])
        accYInterpolated = np.interp(accTimeStampInterp, self.data['timestamp_ns'], self.data['accy'])
        accZInterpolated = np.interp(accTimeStampInterp, self.data['timestamp_ns'], self.data['accz'])

        gyroTimeStampInterp = np.linspace(self.gpsTimeStampInterp[0], self.gpsTimeStampInterp[- 1], self.samples)
        gyroXInterpolated = np.interp(gyroTimeStampInterp, self.data['timestamp_ns'], self.data['gyrx'])
        gyroYInterpolated = np.interp(gyroTimeStampInterp, self.data['timestamp_ns'], self.data['gyry'])
        gyroZInterpolated = np.interp(gyroTimeStampInterp, self.data['timestamp_ns'], self.data['gyrz'])

        magTimeStampInterp = np.linspace(self.gpsTimeStampInterp[0], self.gpsTimeStampInterp[- 1], self.samples)
        magXInterpolated = np.interp(magTimeStampInterp, self.data['timestamp_ns'], self.data['magx'])
        magYInterpolated = np.interp(magTimeStampInterp, self.data['timestamp_ns'], self.data['magy'])
        magZInterpolated = np.interp(magTimeStampInterp, self.data['timestamp_ns'], self.data['magz'])

        self.dt = [(self.gpsTimeStampInterp[i + 1] - self.gpsTimeStampInterp[i]) * 0.000000001 for i in
                   range(len(self.gpsTimeStampInterp) - 1)]
        self.dt.insert(0, 0)

        self.accXAbsolute = []
        self.accYAbsolute = []

        print("Calculating absolute acc values...")
        heading = MadgwickAHRS(sampleperiod=mean(self.dt))
        for i in range(self.samples):
            gyroscope = [gyroZInterpolated[i], gyroYInterpolated[i], gyroXInterpolated[i]]
            accelerometer = [accZInterpolated[i], accYInterpolated[i], accXInterpolated[i]]
            magnetometer = [magZInterpolated[i], magYInterpolated[i], magXInterpolated[i]]
            heading.update(gyroscope, accelerometer, magnetometer)
            ahrs = heading.quaternion.to_euler_angles()
            roll = ahrs[0]
            pitch = ahrs[1]
            yaw = ahrs[2] + (3.0 * (math.pi / 180.0))  # adding magenetic declination
            ACC = np.array([[self.data['accz'][i]], [self.data['accy'][i]], [self.data['accx'][i]]])
            ACCABS = np.linalg.inv(rotaitonMatrix(yaw, pitch, roll)).dot(ACC)
            self.accXAbsolute.append(-1 * ACCABS[0, 0])
            self.accYAbsolute.append(-1 * ACCABS[1, 0])

        self.set_kalman_parameters()
        self.perform_kalman()

        data = {'lat': self.Yfilter,
                'lon': self.Xfilter}

        if self.mapit:
            self.extract_map()
        if self.output_path is not None:
            csv_writter(data=data, path=self.output_path)
        return data

    def perform_kalman(self):
        self.Xfilter = []
        self.Yfilter = []
        print("Running kalman filter...")
        for i in range(len(self.dt)):
            self.X_hat_t, self.P_hat_t = prediction(self.X_hat_t, self.P_t, self.F_t, self.B_t, self.U_t,
                                                    self.Q_t)  # STATE PREDICTION (IMU)
            Z_t = np.array([[self.gpsLongitudeInterpolated[i]], [self.gpsLatitudeInterpolated[i]]])
            Z_t = Z_t.reshape(Z_t.shape[0], -1)
            X_t, self.P_t = update(self.X_hat_t, self.P_hat_t, Z_t, self.R_t, self.H_t)  # MEASUREMENT UPDATE (GPS)
            self.X_hat_t = X_t
            self.P_hat_t = self.P_t
            self.F_t = np.array([[1, 0, self.dt[i], 0], [0, 1, 0, self.dt[i]], [0, 0, 1, 0], [0, 0, 0, 1]])
            self.B_t = np.array(
                [[0.5 * self.dt[i] ** 2, 0, 0, 0], [0, 0.5 * self.dt[i] ** 2, 0, 0], [0, 0, self.dt[i], 0],
                 [0, 0, 0, self.dt[i]]])
            self.U_t = np.array(
                [[self.accXAbsolute[i]], [self.accYAbsolute[i]], [self.accXAbsolute[i]], [self.accYAbsolute[i]]])
            self.Xfilter.append(X_t[0, 0])
            self.Yfilter.append(X_t[1, 0])

    def set_kalman_parameters(self):
        # Transition matrix
        self.F_t = np.array([[1, 0, self.dt[0], 0], [0, 1, 0, self.dt[0]], [0, 0, 1, 0], [0, 0, 0, 1]])
        # Initial State cov
        self.P_t = np.identity(4) * 400
        # Process cov
        self.Q_t = np.array([[1000, 0, 100, 0], [0, 1000, 0, 100], [100, 0, 1000, 0], [0, 100, 0, 1000]]) * 0.65
        # Control matrix
        self.B_t = np.array([[0.5 * self.dt[0] ** 2, 0, 0, 0], [0, 0.5 * self.dt[0] ** 2, 0, 0], [0, 0, self.dt[0], 0],
                             [0, 0, 0, self.dt[0]]])
        # Control vector
        self.U_t = np.array(
            [[self.accXAbsolute[0]], [self.accYAbsolute[0]], [self.accXAbsolute[0]], [self.accYAbsolute[0]]])
        # Measurment Matrix
        self.H_t = np.array([[1, 0, 0, 0], [0, 1, 0, 0]])
        # Measurment cov
        self.R_t = np.identity(2)
        # Initial State
        self.X_hat_t = np.array([[self.gpsLongitudeInterpolated[0]], [self.gpsLatitudeInterpolated[0]], [0], [0]])

    def get_data(self, **kwargs):
        if self.csv_path is not None:
            self.data = get_data_csv(csv_file=self.csv_path)
        else:
            self.data = modify_time(kwargs)

    def extract_map(self):
        # reducing data points to 100 for plotting on google maps
        # dtReduced = np.linspace(self.data['timestamp_ns'][0], self.data['timestamp_ns'][- 1], 100)
        # XfilterInterp = np.interp(dtReduced, self.gpsTimeStampInterp, self.Xfilter)
        # YfilterInterp = np.interp(dtReduced, self.gpsTimeStampInterp, self.Yfilter)

        print("Plotting on google maps...")
        gmap1 = gmplot.GoogleMapPlotter(self.data['lat'][0], self.data['lon'][0], 20,
                                        map_type='hybrid')
        gmap1.apikey = "AIzaSyDvvwPIEA8T9IUxPKaRZ6gp2f6xRtBYICU"
        gmap1.scatter(self.data['lat'], self.data['lon'], color='r', size=0.3, marker=False)
        gmap1.scatter(self.Yfilter, self.Xfilter, color='g', size=0.4, marker=False)
        gmap1.draw(self.output_path+"results_clss.html")

        print("Done...")

    def check_requires(self):
        self.require_list = ['lat', 'lon', 'altitude', 'gyrx', 'gyry', 'gyrz', 'accx', 'accy', 'accz',
                             'magx', 'magy', 'magz', 'timestamp_ns']

        for key in self.require_list:
            if key not in self.data.keys():
                print(f'"{key}" is not exist in data. Process is being continuing without kalman filter')
                return False

        if len(self.data['accx']) == 0:
            print(
                'accx is must bigger than 0 to perform kalman filter. Process is being continuing without kalman filter')
            return False
        return True


def parser():
    # Define the command line arguments
    parser = argparse.ArgumentParser(
        description='kalmangps is an advanced GPS correction tool that utilizes sensor data, including magnetometer, gyroscope, accelerometer, and GPS information, to provide accurate and corrected latitude and longitude coordinates. This application is particularly useful for scenarios where precise location data is essential, such as navigation systems, robotics, or any application requiring accurate positioning.')
    parser.add_argument('-I', '--input_path', type=str,
                        help='Path to the input data file containing sensor information (magnetometer, gyro, ACC, timestamp_ns, and GPS data).')
    parser.add_argument('-O', '--output_path', type=str,
                        help='Path to save the corrected latitude and longitude data as csv file.')
    parser.add_argument('-M', '--mapit', action='store_true',
                        help='Flag to visualize the corrected data on a map. If specified, the application will generate a map displaying the corrected GPS coordinates.')

    # Parse the command line arguments
    args = parser.parse_args()

    return args


def main():
    args = parser()
    klmn = Kalman(csv_path=args.input_path, output_path=args.output_path, mapit=args.mapit)
    output = klmn()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    Copyright (c) 2015 Jonas Böer, jonas.boeer@student.kit.edu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import numpy as np
import numbers


class Quaternion:
    """
    A simple class implementing basic quaternion arithmetic.
    """
    def __init__(self, w_or_q, x=None, y=None, z=None):
        """
        Initializes a Quaternion object
        :param w_or_q: A scalar representing the real part of the quaternion, another Quaternion object or a
                    four-element array containing the quaternion values
        :param x: The first imaginary part if w_or_q is a scalar
        :param y: The second imaginary part if w_or_q is a scalar
        :param z: The third imaginary part if w_or_q is a scalar
        """
        self._q = np.array([1, 0, 0, 0])

        if x is not None and y is not None and z is not None:
            w = w_or_q
            q = np.array([w, x, y, z])
        elif isinstance(w_or_q, Quaternion):
            q = np.array(w_or_q.q)
        else:
            q = np.array(w_or_q)
            if len(q) != 4:
                raise ValueError("Expecting a 4-element array or w x y z as parameters")

        self._set_q(q)

    # Quaternion specific interfaces

    def conj(self):
        """
        Returns the conjugate of the quaternion
        :rtype : Quaternion
        :return: the conjugate of the quaternion
        """
        return Quaternion(self._q[0], -self._q[1], -self._q[2], -self._q[3])

    def to_angle_axis(self):
        """
        Returns the quaternion's rotation represented by an Euler angle and axis.
        If the quaternion is the identity quaternion (1, 0, 0, 0), a rotation along the x axis with angle 0 is returned.
        :return: rad, x, y, z
        """
        if self[0] == 1 and self[1] == 0 and self[2] == 0 and self[3] == 0:
            return 0, 1, 0, 0
        rad = np.arccos(self[0]) * 2
        imaginary_factor = np.sin(rad / 2)
        if abs(imaginary_factor) < 1e-8:
            return 0, 1, 0, 0
        x = self._q[1] / imaginary_factor
        y = self._q[2] / imaginary_factor
        z = self._q[3] / imaginary_factor
        return rad, x, y, z

    @staticmethod
    def from_angle_axis(rad, x, y, z):
        s = np.sin(rad / 2)
        return Quaternion(np.cos(rad / 2), x*s, y*s, z*s)

    def to_euler_angles(self):
        pitch = np.arcsin(2 * self[1] * self[2] + 2 * self[0] * self[3])
        if np.abs(self[1] * self[2] + self[3] * self[0] - 0.5) < 1e-8:
            roll = 0
            yaw = 2 * np.arctan2(self[1], self[0])
        elif np.abs(self[1] * self[2] + self[3] * self[0] + 0.5) < 1e-8:
            roll = -2 * np.arctan2(self[1], self[0])
            yaw = 0
        else:
            roll = np.arctan2(2 * self[0] * self[1] - 2 * self[2] * self[3], 1 - 2 * self[1] ** 2 - 2 * self[3] ** 2)
            yaw = np.arctan2(2 * self[0] * self[2] - 2 * self[1] * self[3], 1 - 2 * self[2] ** 2 - 2 * self[3] ** 2)
        return roll, pitch, yaw

    def to_euler123(self):
        roll = np.arctan2(-2*(self[2]*self[3] - self[0]*self[1]), self[0]**2 - self[1]**2 - self[2]**2 + self[3]**2)
        pitch = np.arcsin(2*(self[1]*self[3] + self[0]*self[1]))
        yaw = np.arctan2(-2*(self[1]*self[2] - self[0]*self[3]), self[0]**2 + self[1]**2 - self[2]**2 - self[3]**2)
        return roll, pitch, yaw

    def __mul__(self, other):
        """
        multiply the given quaternion with another quaternion or a scalar
        :param other: a Quaternion object or a number
        :return:
        """
        if isinstance(other, Quaternion):
            w = self._q[0]*other._q[0] - self._q[1]*other._q[1] - self._q[2]*other._q[2] - self._q[3]*other._q[3]
            x = self._q[0]*other._q[1] + self._q[1]*other._q[0] + self._q[2]*other._q[3] - self._q[3]*other._q[2]
            y = self._q[0]*other._q[2] - self._q[1]*other._q[3] + self._q[2]*other._q[0] + self._q[3]*other._q[1]
            z = self._q[0]*other._q[3] + self._q[1]*other._q[2] - self._q[2]*other._q[1] + self._q[3]*other._q[0]

            return Quaternion(w, x, y, z)
        elif isinstance(other, numbers.Number):
            q = self._q * other
            return Quaternion(q)

    def __add__(self, other):
        """
        add two quaternions element-wise or add a scalar to each element of the quaternion
        :param other:
        :return:
        """
        if not isinstance(other, Quaternion):
            if len(other) != 4:
                raise TypeError("Quaternions must be added to other quaternions or a 4-element array")
            q = self.q + other
        else:
            q = self.q + other.q

        return Quaternion(q)

    # Implementing other interfaces to ease working with the class

    def _set_q(self, q):
        self._q = q

    def _get_q(self):
        return self._q

    q = property(_get_q, _set_q)

    def __getitem__(self, item):
        return self._q[item]

    def __array__(self):
        return self._q



# -*- coding: utf-8 -*-
"""
    Copyright (c) 2015 Jonas Böer, jonas.boeer@student.kit.edu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import warnings
import numpy as np
from numpy.linalg import norm
from .quaternion import Quaternion


class MadgwickAHRS:
    samplePeriod = 1/256
    quaternion = Quaternion(1, 0, 0, 0)
    beta = 1

    def __init__(self, sampleperiod=None, quaternion=None, beta=None):
        """
        Initialize the class with the given parameters.
        :param sampleperiod: The sample period
        :param quaternion: Initial quaternion
        :param beta: Algorithm gain beta
        :return:
        """
        if sampleperiod is not None:
            self.samplePeriod = sampleperiod
        if quaternion is not None:
            self.quaternion = quaternion
        if beta is not None:
            self.beta = beta

    def update(self, gyroscope, accelerometer, magnetometer):
        """
        Perform one update step with data from a AHRS sensor array
        :param gyroscope: A three-element array containing the gyroscope data in radians per second.
        :param accelerometer: A three-element array containing the accelerometer data. Can be any unit since a normalized value is used.
        :param magnetometer: A three-element array containing the magnetometer data. Can be any unit since a normalized value is used.
        :return:
        """
        q = self.quaternion

        gyroscope = np.array(gyroscope, dtype=float).flatten()
        accelerometer = np.array(accelerometer, dtype=float).flatten()
        magnetometer = np.array(magnetometer, dtype=float).flatten()

        # Normalise accelerometer measurement
        if norm(accelerometer) == 0:
            warnings.warn("accelerometer is zero")
            return
        accelerometer /= norm(accelerometer)

        # Normalise magnetometer measurement
        if norm(magnetometer) == 0:
            warnings.warn("magnetometer is zero")
            return
        magnetometer /= norm(magnetometer)

        h = q * (Quaternion(0, magnetometer[0], magnetometer[1], magnetometer[2]) * q.conj())
        b = np.array([0, norm(h[1:3]), 0, h[3]])

        # Gradient descent algorithm corrective step
        f = np.array([
            2*(q[1]*q[3] - q[0]*q[2]) - accelerometer[0],
            2*(q[0]*q[1] + q[2]*q[3]) - accelerometer[1],
            2*(0.5 - q[1]**2 - q[2]**2) - accelerometer[2],
            2*b[1]*(0.5 - q[2]**2 - q[3]**2) + 2*b[3]*(q[1]*q[3] - q[0]*q[2]) - magnetometer[0],
            2*b[1]*(q[1]*q[2] - q[0]*q[3]) + 2*b[3]*(q[0]*q[1] + q[2]*q[3]) - magnetometer[1],
            2*b[1]*(q[0]*q[2] + q[1]*q[3]) + 2*b[3]*(0.5 - q[1]**2 - q[2]**2) - magnetometer[2]
        ])
        j = np.array([
            [-2*q[2],                  2*q[3],                  -2*q[0],                  2*q[1]],
            [2*q[1],                   2*q[0],                  2*q[3],                   2*q[2]],
            [0,                        -4*q[1],                 -4*q[2],                  0],
            [-2*b[3]*q[2],             2*b[3]*q[3],             -4*b[1]*q[2]-2*b[3]*q[0], -4*b[1]*q[3]+2*b[3]*q[1]],
            [-2*b[1]*q[3]+2*b[3]*q[1], 2*b[1]*q[2]+2*b[3]*q[0], 2*b[1]*q[1]+2*b[3]*q[3],  -2*b[1]*q[0]+2*b[3]*q[2]],
            [2*b[1]*q[2],              2*b[1]*q[3]-4*b[3]*q[1], 2*b[1]*q[0]-4*b[3]*q[2],  2*b[1]*q[1]]
        ])
        step = j.T.dot(f)
        step /= norm(step)  # normalise step magnitude

        # Compute rate of change of quaternion
        qdot = (q * Quaternion(0, gyroscope[0], gyroscope[1], gyroscope[2])) * 0.5 - self.beta * step.T

        # Integrate to yield quaternion
        q += qdot * self.samplePeriod
        self.quaternion = Quaternion(q / norm(q))  # normalise quaternion

    def update_imu(self, gyroscope, accelerometer):
        """
        Perform one update step with data from a IMU sensor array
        :param gyroscope: A three-element array containing the gyroscope data in radians per second.
        :param accelerometer: A three-element array containing the accelerometer data. Can be any unit since a normalized value is used.
        """
        q = self.quaternion

        gyroscope = np.array(gyroscope, dtype=float).flatten()
        accelerometer = np.array(accelerometer, dtype=float).flatten()

        # Normalise accelerometer measurement
        if norm(accelerometer) == 0:
            warnings.warn("accelerometer is zero")
            return
        accelerometer /= norm(accelerometer)

        # Gradient descent algorithm corrective step
        f = np.array([
            2*(q[1]*q[3] - q[0]*q[2]) - accelerometer[0],
            2*(q[0]*q[1] + q[2]*q[3]) - accelerometer[1],
            2*(0.5 - q[1]**2 - q[2]**2) - accelerometer[2]
        ])
        j = np.array([
            [-2*q[2], 2*q[3], -2*q[0], 2*q[1]],
            [2*q[1], 2*q[0], 2*q[3], 2*q[2]],
            [0, -4*q[1], -4*q[2], 0]
        ])
        step = j.T.dot(f)
        step /= norm(step)  # normalise step magnitude

        # Compute rate of change of quaternion
        qdot = (q * Quaternion(0, gyroscope[0], gyroscope[1], gyroscope[2])) * 0.5 - self.beta * step.T

        # Integrate to yield quaternion
        q += qdot * self.samplePeriod
        self.quaternion = Quaternion(q / norm(q))  # normalise quaternion



from math import *
import numpy as np
import pandas as pd

# Notation used coming from: https://www.bzarg.com/p/how-a-kalman-filter-works-in-pictures/
def prediction(X_hat_t_1, P_t_1, F_t, B_t, U_t, Q_t):
    X_hat_t = F_t.dot(X_hat_t_1) + (B_t.dot(U_t))
    P_t = np.diag(np.diag(F_t.dot(P_t_1).dot(F_t.transpose()))) + Q_t
    return X_hat_t, P_t
    
def update(X_hat_t, P_t, Z_t, R_t, H_t):
    K_prime = P_t.dot(H_t.transpose()).dot(np.linalg.inv(H_t.dot(P_t).dot(H_t.transpose()) + R_t))
    X_t = X_hat_t + K_prime.dot(Z_t - H_t.dot(X_hat_t))
    P_t = P_t - K_prime.dot(H_t).dot(P_t)
    return X_t, P_t

def rotaitonMatrix(heading, attitude, bank):
    '''
    :returns: rotation array in numpy format
    [m00 m01 m02]
    [m10 m11 m12]
    [m20 m21 m22]
    '''
    ch = cos(heading)
    sh = sin(heading)
    ca = cos(attitude)
    sa = sin(attitude)
    cb = cos(bank)
    sb = sin(bank)
    m00 = ch * ca
    m01 = sh * sb - ch * sa * cb
    m02 = ch * sa * sb + sh * cb
    m10 = sa
    m11 = ca * cb
    m12 = -ca * sb
    m20 = -sh * ca
    m21 = sh * sa * cb + ch * sb
    m22 = -sh * sa * sb + ch * cb
    return np.array([[m00, m01, m02], [m10, m11, m12], [m20, m21, m22]])
    
def getDistance(lat1, lon1, lat2, lon2):
    '''
    reference: http://code.activestate.com/recipes/577594-gps-distance-and-bearing-between-two-gps-points/
    '''
    R = 6371.0
    lat1 = radians(lat1)
    lon1 = radians(lon1)
    lat2 = radians(lat2)
    lon2 = radians(lon2)
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))
    distance = R * c
    return distance * 1000.0

def getBearing(lat1,lon1,lat2,lon2):
    '''
    reference: http://code.activestate.com/recipes/577594-gps-distance-and-bearing-between-two-gps-points/
    '''
    dLon = lon2 - lon1
    y = sin(dLon) * cos(lat2)
    x = cos(lat1) * sin(lat2) \
        - sin(lat1) * cos(lat2) * cos(dLon)
    return atan2(y, x)

def csv_writter(data,path):
    try:
        df = pd.DataFrame(data)
        df.to_csv(path+'output.csv', index=False, header=True)
        print("Data successfully saved.")
    except:
        print("Data could not save.")




import io
import os
import re

import setuptools


def get_long_desc():
    base_dir = os.path.abspath(os.path.dirname(__file__))
    with io.open(os.path.join(base_dir, "README.md"), encoding = "utf-8") as f:
        return f.read()

def get_requirements():
    with open("requirements.txt", encoding = "utf-8") as f:
        return f.read().splitlines()

def get_version():
    cwd = os.path.abspath(os.path.dirname(__file__))
    current_version = os.path.join(cwd, "kalmangps", "components", "version.py")
    with io.open(current_version, encoding = "utf-8") as f:
        return re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', f.read(), re.M).group(1)

setuptools.setup(
    name = "kalmangps",
    version = get_version(),
    author = "Mapilio",
    description = "Kalman Filter for GPS Metadata Correction",
    long_description = get_long_desc(),
    long_description_content_type='text/markdown',
    url = "https://github.com/mapilio/KalmanGps",
    license='MIT License',
    python_requires='>=3.6',
    install_requires = get_requirements(),
    entry_points = {
        "console_scripts": [
            "kalmangps=kalmangps:__init__"
        ]
    }
)



import pandas as pd


def get_data_csv(csv_file: str = None):
    if csv_file is None:
        print("csv file could not found.")
    print("Reading files...")
    # Read the CSV file into a DataFrame
    df = pd.read_csv(csv_file)

    # Convert the "current_time" column to datetime objects
    df['current_time'] = pd.to_datetime(df['current_time'])

    # Convert datetime objects to timestamps with nanoseconds
    df['timestamp_ns'] = df['current_time'].astype(int)
    df.set_index('timestamp_ns')

    # Get rows between 'start_row' and 'end_row'
    # df = df.iloc[10:-10]

    # Convert the DataFrame into a dictionary
    data_dict = df.to_dict(orient='list')
    data_dict['timestamp_ns'] = [int(str(row)[:-3]) for row in data_dict['timestamp_ns']]
    return data_dict


def modify_time(data):
    df = pd.DataFrame(data)

    # Convert the "current_time" column to datetime objects
    df['current_time'] = pd.to_datetime(df['current_time'])

    # Convert datetime objects to timestamps with nanoseconds
    df['timestamp_ns'] = df['current_time'].astype(int)
    df.set_index('timestamp_ns')

    # Convert the DataFrame into a dictionary
    data_dict = df.to_dict(orient='list')
    data_dict['timestamp_ns'] = [int(str(row)[:-3]) for row in data_dict['timestamp_ns']]
    return data_dict


def get_data_txt(txt_file):
    # Defining file locations
    gpsDatafileName = "sensor_data/gpsValues.txt"
    accDatafileName = "sensor_data/accelerometerValues.txt"
    gyroDatafileName = "sensor_data/gyroscopeValues.txt"
    magDatafileName = "sensor_data/magneticValues.txt"

    # defining file handles
    gpsValuesFileHandle = open(gpsDatafileName, 'r')
    accValuesFileHandle = open(accDatafileName, 'r')
    gyroValuesFileHandle = open(gyroDatafileName, 'r')
    magValuesFileHandle = open(magDatafileName, 'r')

    GPSDATA, gpsTimeStamp, gpsLatitude, gpsLongitude, gpsAltitude, gpsBearing, gpsSpeed = ([] for i in range(7))
    ACCDATA, accTimeStamp, accX, accY, accZ = ([] for i in range(5))
    GYRODATA, gyroTimeStamp, gyroX, gyroY, gyroZ = ([] for i in range(5))
    MAGDATA, magTimeStamp, magX, magY, magZ = ([] for i in range(5))

    for gpsData in gpsValuesFileHandle:
        gpsData = gpsData.rstrip()
        GPSDATA.append(gpsData.split(','))
    for i in range(6, len(GPSDATA)):  # ignoring erronious data points
        gpsTimeStamp.append(float(GPSDATA[i][0]))
        gpsLatitude.append(float(GPSDATA[i][1]))
        gpsLongitude.append(float(GPSDATA[i][2]))
        gpsBearing.append(float(GPSDATA[i][4]))
    for accData in accValuesFileHandle:
        accData = accData.rstrip()
        ACCDATA.append(accData.split(','))
    for i in range(1, len(ACCDATA)):
        accTimeStamp.append(float(ACCDATA[i][0]))
        accX.append(float(ACCDATA[i][1]))
        accY.append(float(ACCDATA[i][2]))
        accZ.append(float(ACCDATA[i][3]))
    for gyroData in gyroValuesFileHandle:
        gyroData = gyroData.rstrip()
        GYRODATA.append(gyroData.split(','))
    for i in range(1, len(GYRODATA)):
        gyroTimeStamp.append(float(GYRODATA[i][0]))
        gyroX.append(float(GYRODATA[i][1]))
        gyroY.append(float(GYRODATA[i][2]))
        gyroZ.append(float(GYRODATA[i][3]))
    for magData in magValuesFileHandle:
        magData = magData.rstrip()
        MAGDATA.append(magData.split(','))
    for i in range(1, len(MAGDATA)):
        magTimeStamp.append(float(MAGDATA[i][0]))
        magX.append(float(MAGDATA[i][1]))
        magY.append(float(MAGDATA[i][2]))
        magZ.append(float(MAGDATA[i][3]))

    return gpsTimeStamp, gpsLatitude, gpsLongitude, gpsAltitude, gpsBearing, \
        accTimeStamp, accX, accY, accZ, \
        gyroTimeStamp, gyroX, gyroY, gyroZ, \
        magTimeStamp, magX, magY, magZ
